"""
pyjd
====

*pyjd* is a Python wrapper for the JDownloader API.

:copyright: (c) 2022 Philipp Glaum
:license: GPLv2, see LICENSE for more details
"""

__title__ = "pyjd"
__version__ = "2.0-alpha"
__author__ = "Philipp Glaum"
__copyright__ = "Copyright (c) 2022 Philipp Glaum"
__license__ = "GPLv2"
